import pygame, sys
from pygame.locals import *
from math import pi, acos, sqrt, cos, sin

pygame.init()
WIDTH, HEIGHT = 600, 600
window = pygame.display.set_mode((WIDTH, HEIGHT))
clock = pygame.time.Clock()
font0 = pygame.font.Font(None,40)
font1 = pygame.font.Font(None,80)
font2 = pygame.font.Font(None,30)

class Wall():
    def __init__(self, point1, point2):
        d = (point2[0] - point1[0], point2[1] - point1[1])
        self.len1 = sqrt(d[0]**2 + d[1]**2)
        self.len2 = self.len1/2
        self.dir1 = (d[0]/self.len1, d[1]/self.len1)
        self.dir0 = (-1*self.dir1[1], self.dir1[0])
        self.pos = (point1[0] + self.dir1[0]*self.len2, point1[1] + self.dir1[1]*self.len2)
        self.point1 = point1
        self.point2 = point2

    def drawWall(self, surface):
        pygame.draw.line(surface, (255,255,255), self.point1, self.point2)
        pos = [self.pos[0] + 10*self.dir0[0], self.pos[1] + 10*self.dir0[1]]
        pygame.draw.line(surface, (255,255,255), self.pos, pos)

    def collide(self, particle):
        v0 = particle.vel[0]*self.dir0[0] + particle.vel[1]*self.dir0[1]
        v1 = particle.vel[0]*self.dir1[0] + particle.vel[1]*self.dir1[1]
        particle.vel = [v1*self.dir1[0] - v0*self.dir0[0], v1*self.dir1[1] - v0*self.dir0[1]]
        
    def isIn(self, particle):
        re = False
        d0 = (particle.pos[0] - self.pos[0])*self.dir0[0] + (particle.pos[1] - self.pos[1])*self.dir0[1]
        d1 = (particle.pos[0] - self.pos[0])*self.dir1[0] + (particle.pos[1] - self.pos[1])*self.dir1[1]
        if -2 < d0 and d0 < 0 and abs(d1) < self.len2+1:
            re = True
        return re


class Particle():
    def __init__(self, position, velocity, fadeTime):
        self.pos = list(position)
        self.vel = list(velocity)
        self.i = 255
        self.vi = -1*fadeTime

    def move(self, dt):
        self.pos[0] += self.vel[0]
        self.pos[1] += self.vel[1]
        self.i += self.vi
        
    def drawParticle(self, surface):
        pos = (int(self.pos[0]), int(self.pos[1]))
        pygame.draw.circle(window, (self.i,self.i,self.i), pos, 3)
        #surface.set_at(pos, (self.i,0,0))

class Player():
    def __init__(self, position):
        self.pos = position
        self.vel = [0,0]
        self.speed = 2
        self.max_charge = 3
        self.charge = 3

    def move(self, walls):
        v = sqrt(self.vel[0]**2 + self.vel[1]**2)/self.speed
        if v == 0:
            v = 1
        self.pos[0] += self.vel[0]/v
        self.pos[1] += self.vel[1]/v
        for i in walls:
            if i.isIn(self):
                self.pos[0] -= self.vel[0]/v
                self.pos[1] -= self.vel[1]/v
                break

    def drawPlayer(self, surface):
        pos = (int(self.pos[0]), int(self.pos[1]))
        pygame.draw.circle(surface, (255,0,0), pos, 5)

class Object():
    def __init__(self, position, name):
        self.name = name
        self.rect = pygame.Rect(position, (15,15))
        self.color = (255,255,255)
        if name == 'switch':
            self.color = (0,255,0)
        elif name == 'battery':
            self.color = (0,0,255)
        elif name == 'goal':
            self.color = (255,255,0)

    def isIn(self, player):
        return self.rect.collidepoint(player.pos)

    def drawObj(self, surface):
        pygame.draw.rect(surface, self.color, self.rect)
        

class Wave():
    def __init__(self, world1, world2, obj, playerPos):
        self.par = []
        self.world = [world1, world2]
        self.current = 0
        self.walls = world1
        self.showWalls = True
        self.player = Player(list(playerPos))
        self.obj = obj

    def yell(self, position, n, fade):
        da = 2*pi/n
        for i in range(n):
            x = cos(da*i)
            y = sin(da*i)
            tem = Particle((position[0]+x,position[1]+y),(x,y),fade)
            self.par.append(tem)

    def eventHandler(self, event):
        if event.type == KEYDOWN:
            if event.key == K_w or event.key == K_UP:
                self.player.vel[1] += -1
            elif event.key == K_s or event.key == K_DOWN:
                self.player.vel[1] += 1
            elif event.key == K_a or event.key == K_LEFT:
                self.player.vel[0] += -1
            elif event.key == K_d or event.key == K_RIGHT:
                self.player.vel[0] += 1
            elif event.key == K_ESCAPE or event.key == K_p:
                menuPause()
            elif event.key == K_SPACE and self.current%2 == 1 and self.player.charge > 0:
                self.yell(self.player.pos, 100,0.5)
                self.player.charge -= 1
            elif event.key == K_e:
                for i in self.obj:
                    if i.isIn(self.player):
                        if i.name == 'switch':
                            self.current += 1
                            self.walls = self.world[self.current%2]
                            self.showWalls = not self.showWalls
                            self.par = []
                        elif i.name == 'battery':
                            self.player.charge = self.player.max_charge
                        elif i.name == 'goal':
                            global currentLevel
                            if currentLevel+1 == len(levels):
                                endG()
                            else:
                                currentLevel += 1
                        break
        elif event.type == KEYUP:
            if event.key == K_w or event.key == K_UP:
                self.player.vel[1] += 1
            elif event.key == K_s or event.key == K_DOWN:
                self.player.vel[1] += -1
            elif event.key == K_a or event.key == K_LEFT:
                self.player.vel[0] += 1
            elif event.key == K_d or event.key == K_RIGHT:
                self.player.vel[0] += -1
        
    def drawWave(self, surface):
        if self.current%2 == 0:
            surface.fill((125,125,125))
        else:
            surface.fill((0,0,0))
        if self.showWalls:
            for i in self.walls:
                i.drawWall(surface)
        for i in self.obj:
            i.drawObj(surface)
        self.player.move(self.walls)
        self.player.drawPlayer(surface)
        text = font0.render('Shots: {}'.format(self.player.charge), True, (255,255,255))
        surface.blit(text, (100,550))
        for i in self.par:
            i.move(1)
            if (i.i <= 20):
                self.par.remove(i)
            else:
                i.drawParticle(surface)
                for w in self.walls:
                    if w.isIn(i):
                        w.collide(i)

def reset(mainM, lvl = 0):
    global levels, currentLevel, menu
    levels = [Wave(wL0, wD0, ob0, (145,446)),
          Wave(wL1, wD1, ob1, (84,163)),
          Wave(wL2, wD2, ob2, (337, 294))]
    currentLevel = lvl
    menu = mainM

def menuScreen(surface):
    global menu
    dim = font0.size('Test')[1]+10
    buttons = [[pygame.Rect((300-80,300), (161, dim)), 'Start Game'],
               [pygame.Rect((300-89,350), (176, dim)), 'Level Select'],
               [pygame.Rect((300-84,400), (168, dim)), 'How to Play'],
               [pygame.Rect((300-35,450), (69, dim)), 'Quit']]
    while menu:
        clock.tick(60)
        for event in pygame.event.get():
            if event.type == QUIT:
                pygame.quit()
                sys.exit()
            elif event.type == pygame.MOUSEBUTTONDOWN:
                for i in buttons:
                    if i[0].collidepoint(pygame.mouse.get_pos()):
                        press = buttons.index(i)
                        if press == 0:
                            menu = False
                        elif press == 1:
                            menuLevel(surface)
                        elif press == 2:
                            howTo(surface)
                        elif press == 3:
                            pygame.quit()
                            sys.exit()
        surface.fill((0,0,0))
        text = font1.render('Through the Dark', True, (255,255,255))
        surface.blit(text, (300-237,100))
        for i in buttons:
            pygame.draw.rect(surface, (125,125,125), i[0])
            text = font0.render(i[1], True, (255,255,255))
            surface.blit(text, i[0].move((5,5)))
        pygame.display.update()

def menuPause(surface = window):
    menu = True
    dim = font0.size('Test')[1]+10
    buttons = [[pygame.Rect((20,300), (118, dim)), 'Resume'],
               [pygame.Rect((20,350), (168, dim)), 'How to Play'],
               [pygame.Rect((20,400), (84, dim)), 'Reset'],
               [pygame.Rect((20,450), (159, dim)), 'Main Menu']]
    #for i in buttons:
    #    print font0.size(i[1])
    while menu:
        clock.tick(60)
        for event in pygame.event.get():
            if event.type == QUIT:
                pygame.quit()
                sys.exit()
            elif event.type == pygame.MOUSEBUTTONDOWN:
                for i in buttons:
                    if i[0].collidepoint(pygame.mouse.get_pos()):
                        press = buttons.index(i)
                        if press == 0:
                            menu = False
                        elif press == 1:
                            howTo(surface)
                        elif press == 2:
                            reset(False, currentLevel)
                            menu = False
                        elif press == 3:
                            reset(True)
                            menu = False
            elif event.type == KEYDOWN:
                if event.key == K_p or event.key == K_ESCAPE:
                    menu = False
        surface.fill((125,125,125))
        pygame.draw.rect(surface, (0,0,0), (0,0,WIDTH/3,HEIGHT))
        text = font1.render('Pause', True, (255,255,255))
        surface.blit(text, (10,100))
        for i in buttons:
            pygame.draw.rect(surface, (150,150,150), i[0])
            text = font0.render(i[1], True, (255,255,255))
            surface.blit(text, i[0].move((5,5)))
        pygame.display.update()

def howTo(surface):
    menu = True
    dim = font0.size('Back')[1]+10
    button = [pygame.Rect((20,20), (76, dim)), 'Back']
    s = ['(w,a,s,d) or arrows to move.',
         '(p,Esc) to pause menu.',
         '(e) to interact.',
         '(SPACE) to shot (only in the dark).',
         'Green square -> turn on/off the Dark.',
         'Blue square -> Recharge your shots.',
         'Yellow square -> The Goal.']
    while menu:
        clock.tick(60)
        for event in pygame.event.get():
            if event.type == QUIT:
                pygame.quit()
                sys.exit()
            elif event.type == pygame.MOUSEBUTTONDOWN:
                if button[0].collidepoint(pygame.mouse.get_pos()):
                    menu = False
            elif event.type == KEYDOWN:
                if event.key == K_ESCAPE:
                    menu = False
        surface.fill((125,125,125))
        pygame.draw.rect(surface, (0,0,0), (0,200,WIDTH,HEIGHT-200))
        text = font1.render('How to Play', True, (255,255,255))
        surface.blit(text, (10,100))
        pygame.draw.rect(surface, (150,150,150), button[0])
        text = font0.render(button[1], True, (255,255,255))
        surface.blit(text, button[0].move((5,5)))
        for i in s:
            text = font0.render(i, True, (255,255,255))
            surface.blit(text, (20,250 + 50*s.index(i)))
        pygame.display.update()

def menuLevel(surface):
    global currentLevel
    menu = True
    dim = font0.size('Test')[1]+10
    lvl = range(len(levels))
    button = pygame.Rect((20,20), (76, dim))
    buttons = []
    for i in lvl:
        buttons.append(pygame.Rect((20,200 + dim*i), (100, dim-5)))
    #for i in buttons:
    #    print font0.size(i[1])
    while menu:
        clock.tick(60)
        for event in pygame.event.get():
            if event.type == QUIT:
                pygame.quit()
                sys.exit()
            elif event.type == pygame.MOUSEBUTTONDOWN:
                if button.collidepoint(pygame.mouse.get_pos()):
                    menu = False
                for i in buttons:
                    if i.collidepoint(pygame.mouse.get_pos()):
                        reset(False, buttons.index(i))
                        menu = False
            elif event.type == KEYDOWN:
                if event.key == K_p or event.key == K_ESCAPE:
                    menu = False
        surface.fill((125,125,125))
        pygame.draw.rect(surface, (0,0,0), (0,0,WIDTH/3,HEIGHT))
        text = font1.render('Select Level', True, (255,255,255))
        surface.blit(text, (10,100))
        pygame.draw.rect(surface, (150,150,150), button)
        text = font0.render('Back', True, (255,255,255))
        surface.blit(text, button.move((5,5)))
        for i in buttons:
            pygame.draw.rect(surface, (150,150,150), i)
            text = font0.render(str(buttons.index(i)), True, (255,255,255))
            surface.blit(text, i.move((5,5)))
        pygame.display.update()

def endG(surface = window):
    menu = True
    s = ['Press any key to return to the main menu.']
    while menu:
        clock.tick(60)
        for event in pygame.event.get():
            if event.type == QUIT:
                pygame.quit()
                sys.exit()
            elif event.type == KEYDOWN:
                menu = False
        surface.fill((125,125,125))
        pygame.draw.rect(surface, (0,0,0), (0,200,WIDTH,HEIGHT-200))
        text = font1.render('Thanks for Playing', True, (255,255,255))
        surface.blit(text, (10,100))
        for i in s:
            text = font0.render(i, True, (255,255,255))
            surface.blit(text, (20,250 + 50*s.index(i)))
        pygame.display.update()
    reset(True)

def introG(surface = window):
    menu = True    
    s = ['There are two worlds, one is full of light,',
         'and other is just darkness, to find the exit',
         'you will have to travel between these worlds,',
         'but do not worry, since you will have',
         'a special object to see in the dark.',
         '',
         'Press any key to cotinue.']
    while menu:
        clock.tick(60)
        for event in pygame.event.get():
            if event.type == QUIT:
                pygame.quit()
                sys.exit()
            elif event.type == KEYDOWN:
                menu = False
        surface.fill((125,125,125))
        pygame.draw.rect(surface, (0,0,0), (0,200,WIDTH,HEIGHT-200))
        text = font1.render('Through the Dark', True, (255,255,255))
        surface.blit(text, (10,100))
        for i in s:
            text = font2.render(i, True, (255,255,255))
            surface.blit(text, (20,250 + 50*s.index(i)))
        pygame.display.update()
    
wL0 = [Wall((100,100),(500,100)),
      Wall((500,100),(500,300)),
      Wall((500,300),(100,300)),
      Wall((100,300),(100,100)),
      Wall((100,400),(500,400)),
      Wall((500,400),(500,500)),
      Wall((500,500),(100,500)),
      Wall((100,500),(100,400))]

wD0 = [Wall((400,100),(500,100)),
      Wall((500,100),(500,300)),
      Wall((500,300),(400,300)),
      Wall((400,300),(400,400)),
      Wall((400,400),(500,400)),
      Wall((500,400),(500,500)),
      Wall((500,500),(300,500)),
      Wall((300,500),(300,200)),
      Wall((300,200),(400,200)),
      Wall((400,200),(400,100))]

ob0 = [Object((400,100), 'switch'),
      Object((400,400), 'switch'),
       Object((148,198), 'goal')]

wL1 = [Wall((33, 371),(20, 128)),
      Wall((20, 128),(157, 130)),
      Wall((157, 130),(158, 368)),
      Wall((158, 368),(457, 363)),
      Wall((457, 363),(450, 464)),
      Wall((450, 464),(579, 459)),
      Wall((579, 459),(575, 574)),
      Wall((575, 574),(386, 572)),
      Wall((386, 572),(90, 493)),
      Wall((90, 493),(33, 371)),
      Wall((23, 91),(17, 17)),
      Wall((17, 17),(548, 19)),
      Wall((548, 19),(545, 306)),
      Wall((545, 306),(435, 315)),
      Wall((435, 315),(415, 93)),
      Wall((415, 93),(23, 91))]

wD1 = [Wall((517, 587),(462, 561)),
      Wall((462, 561),(230, 559)),
      Wall((230, 559),(235, 435)),
      Wall((235, 435),(100, 427)),
      Wall((100, 427),(96, 114)),
      Wall((96, 114),(10, 104)),
      Wall((10, 104),(13, 13)),
      Wall((13, 13),(148, 12)),
      Wall((148, 12),(171, 103)),
      Wall((171, 103),(166, 361)),
      Wall((166, 361),(280, 367)),
      Wall((280, 367),(314, 431)),
      Wall((314, 431),(301, 497)),
      Wall((301, 497),(543, 478)),
      Wall((543, 478),(583, 546)),
      Wall((583, 546),(517, 587))]

ob1 = [Object((528, 540), "switch"),
      Object((47, 47), "switch"),
      Object((80, 160), "battery"),
      Object((494,265), 'goal')]

wL2 = [Wall((379, 341),(202, 339)),
      Wall((202, 339),(206, 242)),
      Wall((206, 242),(387, 248)),
      Wall((387, 248),(379, 341)),
       Wall((562, 341),(460, 349)),
      Wall((460, 349),(464, 252)),
      Wall((464, 252),(556, 259)),
      Wall((556, 259),(562, 341))]

wD2 = [Wall((356, 419),(415, 358)),
      Wall((415, 358),(416, 300)),
      Wall((416, 300),(502, 304)),
      Wall((502, 304),(501, 378)),
      Wall((501, 378),(421, 464)),
      Wall((421, 464),(83, 446)),
      Wall((83, 446),(105, 168)),
      Wall((105, 168),(291, 170)),
      Wall((291, 170),(258, 280)),
      Wall((258, 280),(184, 288)),
      Wall((184, 288),(141, 398)),
      Wall((141, 398),(356, 419))]

ob2 =[Object((234, 261), "switch"),
      Object((479, 319), "switch"),
      Object((231, 302), "battery"),
      Object((522, 277), "goal")]
'''
wL3 = 

wD3 = 

ob3 =

wL4 = 

wD4 = 

ob4 =

wL5 = 

wD5 = 

ob5 =
'''
levels = [Wave(wL0, wD0, ob0, (145,446)),
          Wave(wL1, wD1, ob1, (84,163)),
          Wave(wL2, wD2, ob2, (337, 294))]
currentLevel = 0

introG(window)
menu = True

pygame.display.set_caption('Through the Dark')
while True:
    clock.tick(60)
    if menu:
        menuScreen(window)
    for event in pygame.event.get():
        if event.type == QUIT:
            pygame.quit()
            sys.exit()
        elif event.type == pygame.MOUSEBUTTONDOWN:
            #levels[currentLevel].yell(pygame.mouse.get_pos(), 100,0.5)
            print pygame.mouse.get_pos()
        else:
            levels[currentLevel].eventHandler(event)

    levels[currentLevel].drawWave(window)
    pygame.display.update()
    #pygame.display.set_caption('FPS: {}'.format(clock.get_fps()))

